@@ -2,6 +2,7 @@ use std::{fmt, str::FromStr};
 
 use crate::{
     buffer::{BufferHandle, CharDisplayDistances},
+    buffer_position::BufferPositionIndex,
     buffer_view::{BufferViewCollection, BufferViewHandle},
     editor::Editor,
     events::{EditorEvent, EditorEventQueue},
@@ -65,7 +66,7 @@ pub struct Client {
     handle: ClientHandle,
 
     pub viewport_size: (u16, u16),
-    pub scroll: (u32, u32),
+    pub scroll: (BufferPositionIndex, BufferPositionIndex),
     pub height: u16,
     pub navigation_history: NavigationHistory,
 
@@ -149,30 +150,40 @@ impl Client {
     }
 
     pub fn update_view(&mut self, editor: &Editor, picker_height: u16) {
-        fn calculate_scroll(this: &Client, editor: &Editor) -> Option<(u32, u32)> {
+        fn calculate_scroll(
+            this: &Client,
+            editor: &Editor,
+        ) -> Option<(BufferPositionIndex, BufferPositionIndex)> {
             let buffer_view = editor.buffer_views.get(this.buffer_view_handle?)?;
             let buffer = editor.buffers.get(buffer_view.buffer_handle)?.content();
 
             let position = buffer_view.cursors.main_cursor().position;
-            let line_index = position.line_index as u32;
+            let line_index = position.line_index;
             let line = buffer.line_at(line_index as _).as_str();
-            let column_index = position.column_byte_index as u32;
+            let column_index = position.column_byte_index;
 
-            let width = this.viewport_size.0 as u32;
-            let height = this.height as u32;
+            let width = this.viewport_size.0 as BufferPositionIndex;
+            let height = this.height as BufferPositionIndex;
             let half_height = height / 2;
 
             let (mut scroll_x, mut scroll_y) = this.scroll;
 
             if column_index < scroll_x {
                 scroll_x = column_index
-            } else if let Some(d) =
-                CharDisplayDistances::new(&line[..column_index as usize], editor.config.tab_size)
+            } else {
+                let index = column_index as usize;
+                let (width, text) = match line[index..].chars().next() {
+                    Some(c) => (width, &line[..index + c.len_utf8()]),
+                    None => (width - 1, line),
+                };
+
+                if let Some(d) = CharDisplayDistances::new(text, editor.config.tab_size)
                     .rev()
-                    .take_while(|d| d.distance < width as _)
+                    .take_while(|d| d.distance <= width as _)
                     .last()
-            {
-                scroll_x = scroll_x.max(d.char_index as _);
+                {
+                    scroll_x = scroll_x.max(d.char_index as _);
+                }
             }
 
             if line_index < scroll_y.saturating_sub(half_height) {
@@ -271,3 +282,4 @@ impl ClientManager {
         self.clients.iter_mut().filter(|c| c.active)
     }
 }
+
