@@ -139,7 +139,7 @@ impl Editor {
 
             buffered_keys: BufferedKeys::default(),
             recording_macro: None,
-            registers: RegisterCollection::default(),
+            registers: RegisterCollection::new(),
             read_line: ReadLine::default(),
             picker: Picker::default(),
             string_pool: StringPool::default(),
@@ -163,7 +163,8 @@ impl Editor {
         } else {
             let path = path.strip_prefix(&self.current_directory).unwrap_or(path);
             let buffer = self.buffers.add_new();
-            buffer.set_path(path);
+            buffer.path.clear();
+            buffer.path.push(path);
             buffer.capabilities = BufferCapabilities::text();
             let _ = buffer.discard_and_reload_from_file(&mut self.word_database, &mut self.events);
 
@@ -280,7 +281,7 @@ impl Editor {
     ) -> EditorControlFlow {
         match event {
             ClientEvent::Command(client_handle, commands) => {
-                let result = CommandManager::eval_commands_then_output(
+                let result = CommandManager::eval_and_then_output(
                     self,
                     platform,
                     clients,
@@ -375,7 +376,6 @@ impl Editor {
         platform: &mut Platform,
         clients: &mut ClientManager,
         tag: ProcessTag,
-        success: bool,
     ) {
         match tag {
             ProcessTag::Buffer(index) => {
@@ -386,7 +386,7 @@ impl Editor {
                 lsp::ClientManager::on_process_exit(self, client_handle)
             }
             ProcessTag::Command(index) => {
-                CommandManager::on_process_exit(self, platform, clients, index, success)
+                CommandManager::on_process_exit(self, platform, clients, index)
             }
         }
 
@@ -410,6 +410,7 @@ impl Editor {
                     &EditorEvent::BufferLoad { handle } => {
                         if let Some(buffer) = self.buffers.get_mut(handle) {
                             buffer.refresh_syntax(&self.syntaxes);
+                            self.buffer_views.on_buffer_load(buffer);
                         }
                     }
                     &EditorEvent::BufferInsertText { handle, range, .. } => {
@@ -459,3 +460,4 @@ impl Editor {
         }
     }
 }
+
