@@ -2,12 +2,13 @@ use std::{cmp::Ordering, fmt::Write, path::Path};
 
 use crate::{
     buffer::{find_path_and_position_at, parse_path_and_position, BufferContent},
-    buffer_position::{BufferPosition, BufferRange},
+    buffer_position::{BufferPosition, BufferPositionIndex, BufferRange},
     buffer_view::{BufferViewHandle, CursorMovement, CursorMovementKind},
     client::Client,
     cursor::{Cursor, CursorCollection},
     editor::{Editor, EditorControlFlow, KeysIterator},
     editor_utils::{hash_bytes, MessageKind},
+    help::HELP_PREFIX,
     lsp,
     mode::{picker, read_line, Mode, ModeContext, ModeKind, ModeOperation, ModeState},
     navigation_history::{NavigationDirection, NavigationHistory},
@@ -29,7 +30,7 @@ pub struct State {
     is_recording_auto_macro: bool,
     pub count: u32,
     last_copy_hash: u64,
-    last_copy_ranges: Vec<(u32, u32)>,
+    last_copy_ranges: Vec<(BufferPositionIndex, BufferPositionIndex)>,
 }
 
 impl State {
@@ -321,7 +322,10 @@ impl State {
                         cursors.clear();
                         cursors.add(Cursor {
                             anchor: BufferPosition::zero(),
-                            position: BufferPosition::line_col(last_line_index, last_line_len),
+                            position: BufferPosition::line_col(
+                                last_line_index as _,
+                                last_line_len as _,
+                            ),
                         });
                     }
                     Key::Char('(') | Key::Char(')') => {
@@ -339,6 +343,7 @@ impl State {
                     Key::Char('|') => delimiter_pair(buffer, &mut cursors[..], '|'),
                     Key::Char('"') => delimiter_pair(buffer, &mut cursors[..], '"'),
                     Key::Char('\'') => delimiter_pair(buffer, &mut cursors[..], '\''),
+                    Key::Char('`') => delimiter_pair(buffer, &mut cursors[..], '`'),
                     _ => (),
                 }
 
@@ -356,11 +361,13 @@ impl State {
                         if let Some(range) = range {
                             cursor.anchor = BufferPosition::line_col(
                                 range.from.line_index,
-                                range.from.column_byte_index - left.len_utf8(),
+                                range.from.column_byte_index
+                                    - left.len_utf8() as BufferPositionIndex,
                             );
                             cursor.position = BufferPosition::line_col(
                                 range.to.line_index,
-                                range.to.column_byte_index + right.len_utf8(),
+                                range.to.column_byte_index
+                                    + right.len_utf8() as BufferPositionIndex,
                             );
                         }
                     }
@@ -372,11 +379,13 @@ impl State {
                         if let Some(range) = range {
                             cursor.anchor = BufferPosition::line_col(
                                 range.from.line_index,
-                                range.from.column_byte_index - delimiter.len_utf8(),
+                                range.from.column_byte_index
+                                    - delimiter.len_utf8() as BufferPositionIndex,
                             );
                             cursor.position = BufferPosition::line_col(
                                 range.to.line_index,
-                                range.to.column_byte_index + delimiter.len_utf8(),
+                                range.to.column_byte_index
+                                    + delimiter.len_utf8() as BufferPositionIndex,
                             );
                         }
                     }
@@ -411,7 +420,10 @@ impl State {
                         cursors.clear();
                         cursors.add(Cursor {
                             anchor: BufferPosition::zero(),
-                            position: BufferPosition::line_col(last_line_index, last_line_len),
+                            position: BufferPosition::line_col(
+                                last_line_index as _,
+                                last_line_len as _,
+                            ),
                         });
                     }
                     Key::Char('(') | Key::Char(')') => {
@@ -517,16 +529,16 @@ impl State {
                         for cursor in &mut buffer_view.cursors.mut_guard()[..] {
                             let mut position = cursor.position;
 
-                            let line = buffer.line_at(position.line_index).as_str();
-                            let cursor_char = if position.column_byte_index < line.len() {
-                                match line[position.column_byte_index..].chars().next() {
+                            let line = buffer.line_at(position.line_index as _).as_str();
+                            let cursor_char = if position.column_byte_index < line.len() as _ {
+                                match line[position.column_byte_index as usize..].chars().next() {
                                     Some(c) => c,
                                     None => continue,
                                 }
                             } else {
                                 match line.char_indices().next_back() {
                                     Some((i, c)) => {
-                                        position.column_byte_index = i;
+                                        position.column_byte_index = i as _;
                                         c
                                     }
                                     None => continue,
@@ -538,7 +550,7 @@ impl State {
                                 '[' | ']' => buffer.find_balanced_chars_at(position, '[', ']'),
                                 '{' | '}' => buffer.find_balanced_chars_at(position, '{', '}'),
                                 '<' | '>' => buffer.find_balanced_chars_at(position, '<', '>'),
-                                d @ '|' | d @ '"' | d @ '\'' => {
+                                d @ '|' | d @ '"' | d @ '\'' | d @ '`' => {
                                     buffer.find_delimiter_pair_at(position, d)
                                 }
                                 _ => continue,
@@ -582,21 +594,17 @@ impl State {
                                 continue;
                             }
 
-                            let line = ctx
-                                .editor
-                                .buffers
-                                .get(buffer_handle)?
-                                .content()
-                                .line_at(line_index)
-                                .as_str();
+                            let buffer = ctx.editor.buffers.get(buffer_handle)?;
+
+                            let line = buffer.content().line_at(line_index as _).as_str();
 
                             let from = range.from.column_byte_index;
                             let to = range.to.column_byte_index;
 
                             let (path, position) = if from < to {
-                                parse_path_and_position(&line[from..to])
+                                parse_path_and_position(&line[from as usize..to as usize])
                             } else {
-                                find_path_and_position_at(line, from)
+                                find_path_and_position_at(line, from as _)
                             };
                             let position = match position {
                                 Some(position) => position,
@@ -604,9 +612,26 @@ impl State {
                             };
 
                             path_buf.clear();
+                            if Path::new(path).is_relative() {
+                                if buffer.path.starts_with(HELP_PREFIX) {
+                                    path_buf.push_str(HELP_PREFIX);
+                                } else if let Some(parent) =
+                                    buffer.path.parent().and_then(Path::to_str)
+                                {
+                                    if !parent.is_empty() {
+                                        path_buf.push_str(parent);
+                                        path_buf.push('/');
+                                    }
+                                }
+                            }
                             path_buf.push_str(path);
+
                             let path = Path::new(&path_buf);
-                            if !path.exists() {
+                            if !path.starts_with(HELP_PREFIX) && !path.exists() {
+                                ctx.editor
+                                    .status_bar
+                                    .write(MessageKind::Error)
+                                    .fmt(format_args!("file {:?} does not exist", path));
                                 continue;
                             }
 
@@ -635,6 +660,8 @@ impl State {
                             }
                             jumped = true;
 
+                            ctx.editor.mode.normal_state.movement_kind =
+                                CursorMovementKind::PositionAndAnchor;
                             if let Some(client) = ctx.clients.get_mut(ctx.client_handle) {
                                 client.set_buffer_view_handle(Some(handle), &mut ctx.editor.events);
                             }
@@ -715,21 +742,27 @@ impl State {
                 for cursor in &mut buffer_view.cursors.mut_guard()[..] {
                     if cursor.anchor <= cursor.position {
                         cursor.anchor.column_byte_index = 0;
-                        cursor.position.line_index += count as usize;
-                        if cursor.position.line_index <= last_line_index {
+                        cursor.position.line_index += count as BufferPositionIndex;
+                        if cursor.position.line_index <= last_line_index as _ {
                             cursor.position.column_byte_index = 0;
                         } else {
-                            cursor.position.line_index = last_line_index;
-                            cursor.position.column_byte_index =
-                                buffer.line_at(cursor.position.line_index).as_str().len();
+                            cursor.position.line_index = last_line_index as _;
+                            cursor.position.column_byte_index = buffer
+                                .line_at(cursor.position.line_index as _)
+                                .as_str()
+                                .len()
+                                as _;
                         }
                     } else {
                         cursor.anchor.column_byte_index =
-                            buffer.line_at(cursor.anchor.line_index).as_str().len();
-                        if cursor.position.line_index >= count as usize {
-                            cursor.position.line_index -= count as usize;
-                            cursor.position.column_byte_index =
-                                buffer.line_at(cursor.position.line_index).as_str().len();
+                            buffer.line_at(cursor.anchor.line_index as _).as_str().len() as _;
+                        if cursor.position.line_index >= count as _ {
+                            cursor.position.line_index -= count as BufferPositionIndex;
+                            cursor.position.column_byte_index = buffer
+                                .line_at(cursor.position.line_index as _)
+                                .as_str()
+                                .len()
+                                as _;
                         } else {
                             cursor.position.line_index = 0;
                             cursor.position.column_byte_index = 0;
@@ -740,20 +773,19 @@ impl State {
             }
             Key::Char('z') => {
                 let buffer_view = ctx.editor.buffer_views.get(handle)?;
-                let focused_line_index =
-                    buffer_view.cursors.main_cursor().position.line_index as u32;
+                let focused_line_index = buffer_view.cursors.main_cursor().position.line_index;
                 let client = ctx.clients.get_mut(ctx.client_handle)?;
-                let height = client.height as u32;
+                let height = client.height;
 
                 match keys.next(&ctx.editor.buffered_keys) {
                     Key::None => return Some(ModeOperation::Pending),
                     Key::Char('z') => {
                         client.scroll.0 = 0;
-                        client.scroll.1 = focused_line_index.saturating_sub(height / 2);
+                        client.scroll.1 = focused_line_index.saturating_sub((height / 2) as _);
                     }
                     Key::Char('j') => {
                         client.scroll.0 = 0;
-                        client.scroll.1 = focused_line_index.saturating_sub(height);
+                        client.scroll.1 = focused_line_index.saturating_sub(height as _);
                     }
                     Key::Char('k') => {
                         client.scroll.0 = 0;
@@ -771,11 +803,13 @@ impl State {
                     let position = match buffer
                         .lines()
                         .enumerate()
-                        .skip(cursor.position.line_index + 1)
+                        .skip(cursor.position.line_index as usize + 1)
                         .filter(|(_, l)| l.as_str().chars().all(|c| c.is_whitespace()))
                         .nth(state.count.max(1).saturating_sub(1) as _)
                     {
-                        Some((i, line)) => BufferPosition::line_col(i, line.as_str().len()),
+                        Some((i, line)) => {
+                            BufferPosition::line_col(i as _, line.as_str().len() as _)
+                        }
                         None => {
                             let line_index = buffer.line_count() - 1;
                             let column_byte_index = buffer
@@ -783,7 +817,7 @@ impl State {
                                 .as_str()
                                 .find(|c: char| !c.is_whitespace())
                                 .unwrap_or(0);
-                            BufferPosition::line_col(line_index, column_byte_index)
+                            BufferPosition::line_col(line_index as _, column_byte_index as _)
                         }
                     };
                     cursor.position = position;
@@ -802,18 +836,20 @@ impl State {
                         .lines()
                         .enumerate()
                         .rev()
-                        .skip(buffer.line_count() - cursor.position.line_index)
+                        .skip(buffer.line_count() - cursor.position.line_index as usize)
                         .filter(|(_, l)| l.as_str().chars().all(|c| c.is_whitespace()))
                         .nth(state.count.max(1).saturating_sub(1) as _)
                     {
-                        Some((i, line)) => BufferPosition::line_col(i, line.as_str().len()),
+                        Some((i, line)) => {
+                            BufferPosition::line_col(i as _, line.as_str().len() as _)
+                        }
                         None => {
                             let column_byte_index = buffer
                                 .line_at(0)
                                 .as_str()
                                 .find(|c: char| !c.is_whitespace())
                                 .unwrap_or(0);
-                            BufferPosition::line_col(0, column_byte_index)
+                            BufferPosition::line_col(0, column_byte_index as _)
                         }
                     };
                     cursor.position = position;
@@ -891,7 +927,7 @@ impl State {
                 for i in 0..cursor_count {
                     let range = ctx.editor.buffer_views.get(handle)?.cursors[i].to_range();
                     for line_index in range.from.line_index..=range.to.line_index {
-                        let line = buffer.content().line_at(line_index).as_str();
+                        let line = buffer.content().line_at(line_index as _).as_str();
                         let mut indentation_column_index = 0;
 
                         for _ in 0..count {
@@ -913,7 +949,7 @@ impl State {
                         }
                         let range = BufferRange::between(
                             BufferPosition::line_col(line_index, 0),
-                            BufferPosition::line_col(line_index, indentation_column_index),
+                            BufferPosition::line_col(line_index, indentation_column_index as _),
                         );
                         buffer.delete_range(
                             &mut ctx.editor.word_database,
@@ -964,6 +1000,13 @@ impl State {
             Key::Char('c') | Key::Char('C') => match keys.next(&ctx.editor.buffered_keys) {
                 Key::None => return Some(ModeOperation::Pending),
                 Key::Char('c') => {
+                    for cursor in
+                        &mut ctx.editor.buffer_views.get_mut(handle)?.cursors.mut_guard()[..]
+                    {
+                        std::mem::swap(&mut cursor.anchor, &mut cursor.position);
+                    }
+                }
+                Key::Char('l') => {
                     let buffer_view = ctx.editor.buffer_views.get_mut(handle)?;
                     let buffer = ctx.editor.buffers.get(buffer_view.buffer_handle)?.content();
 
@@ -981,14 +1024,14 @@ impl State {
                             cursor.anchor = range.from;
                             cursor.position = BufferPosition::line_col(
                                 range.from.line_index,
-                                buffer.line_at(range.from.line_index).as_str().len(),
+                                buffer.line_at(range.from.line_index as _).as_str().len() as _,
                             );
 
                             for line_index in (range.from.line_index + 1)..range.to.line_index {
-                                let line_len = buffer.line_at(line_index).as_str().len();
+                                let line_len = buffer.line_at(line_index as _).as_str().len();
                                 cursors.add(Cursor {
                                     anchor: BufferPosition::line_col(line_index, 0),
-                                    position: BufferPosition::line_col(line_index, line_len),
+                                    position: BufferPosition::line_col(line_index, line_len as _),
                                 });
                             }
 
@@ -1001,9 +1044,9 @@ impl State {
                             cursor.position = BufferPosition::line_col(range.to.line_index, 0);
 
                             for line_index in (range.from.line_index + 1)..range.to.line_index {
-                                let line_len = buffer.line_at(line_index).as_str().len();
+                                let line_len = buffer.line_at(line_index as _).as_str().len();
                                 cursors.add(Cursor {
-                                    anchor: BufferPosition::line_col(line_index, line_len),
+                                    anchor: BufferPosition::line_col(line_index, line_len as _),
                                     position: BufferPosition::line_col(line_index, 0),
                                 });
                             }
@@ -1011,13 +1054,12 @@ impl State {
                             cursors.add(Cursor {
                                 anchor: BufferPosition::line_col(
                                     range.from.line_index,
-                                    buffer.line_at(range.from.line_index).as_str().len(),
+                                    buffer.line_at(range.from.line_index as _).as_str().len() as _,
                                 ),
                                 position: range.from,
                             });
                         }
                     }
-                    state.movement_kind = CursorMovementKind::PositionOnly;
                 }
                 Key::Char('d') => {
                     let cursors = &mut ctx.editor.buffer_views.get_mut(handle)?.cursors;
@@ -1038,13 +1080,6 @@ impl State {
                     }
                     state.movement_kind = CursorMovementKind::PositionAndAnchor;
                 }
-                Key::Char('o') => {
-                    for cursor in
-                        &mut ctx.editor.buffer_views.get_mut(handle)?.cursors.mut_guard()[..]
-                    {
-                        std::mem::swap(&mut cursor.anchor, &mut cursor.position);
-                    }
-                }
                 Key::Char('j') => {
                     let buffer_view = ctx.editor.buffer_views.get_mut(handle)?;
                     let buffer = ctx.editor.buffers.get(buffer_view.buffer_handle)?;
@@ -1130,13 +1165,15 @@ impl State {
                 let mut text = ctx.editor.string_pool.acquire();
                 copy_text(ctx, handle, &mut text);
                 if !text.is_empty() {
-                    ctx.platform.write_to_clipboard(&text);
+                    ctx.platform
+                        .write_to_clipboard(&ctx.editor.registers, &text);
                 }
                 ctx.editor.string_pool.release(text);
             }
             Key::Char('Y') => {
                 let mut text = ctx.editor.string_pool.acquire();
-                ctx.platform.read_from_clipboard(&mut text);
+                ctx.platform
+                    .read_from_clipboard(&ctx.editor.registers, &mut text);
                 paste_text(ctx, handle, &text);
                 ctx.editor.string_pool.release(text);
                 return None;
@@ -1369,8 +1406,8 @@ fn find_char(ctx: &mut ModeContext, forward: bool) -> Option<()> {
     for cursor in &mut buffer_view.cursors.mut_guard()[..] {
         let (left_chars, right_chars) = buffer
             .content()
-            .line_at(cursor.position.line_index)
-            .chars_from(cursor.position.column_byte_index);
+            .line_at(cursor.position.line_index as _)
+            .chars_from(cursor.position.column_byte_index as _);
 
         let element = match forward {
             false => left_chars
@@ -1385,9 +1422,9 @@ fn find_char(ctx: &mut ModeContext, forward: bool) -> Option<()> {
                 .last(),
         };
         if let Some((i, c)) = element {
-            cursor.position.column_byte_index = i;
+            cursor.position.column_byte_index = i as _;
             if next_ch {
-                cursor.position.column_byte_index += c.len_utf8();
+                cursor.position.column_byte_index += c.len_utf8() as BufferPositionIndex;
             }
 
             if let CursorMovementKind::PositionAndAnchor = state.movement_kind {
