@@ -72,7 +72,7 @@ const MAX_EVENT_COUNT: usize = 1 + 1 + MAX_CLIENT_COUNT + MAX_PROCESS_COUNT;
 const _ASSERT_MAX_EVENT_COUNT_IS_64: [(); 64] = [(); MAX_EVENT_COUNT];
 
 const CLIENT_EVENT_BUFFER_LEN: usize = 32;
-const PIPE_PREFIX: &str = r#"\\.\pipe\"#;
+static PIPE_PREFIX: &str = r#"\\.\pipe\"#;
 
 pub fn main() {
     let args = Args::parse();
@@ -955,10 +955,9 @@ fn run_server(args: Args, pipe_path: &[u16]) -> Result<(), AnyError> {
                                             handle,
                                         })?;
                                     }
-                                    Err(_) => event_sender.send(ApplicationEvent::ProcessExit {
-                                        tag,
-                                        success: false,
-                                    })?,
+                                    Err(_) => {
+                                        event_sender.send(ApplicationEvent::ProcessExit { tag })?
+                                    }
                                 }
                                 break;
                             }
@@ -969,10 +968,7 @@ fn run_server(args: Args, pipe_path: &[u16]) -> Result<(), AnyError> {
                                     let tag = process.tag;
                                     process.kill();
                                     processes[handle.0] = None;
-                                    event_sender.send(ApplicationEvent::ProcessExit {
-                                        tag,
-                                        success: false,
-                                    })?;
+                                    event_sender.send(ApplicationEvent::ProcessExit { tag })?;
                                 }
                             }
                         }
@@ -986,10 +982,7 @@ fn run_server(args: Args, pipe_path: &[u16]) -> Result<(), AnyError> {
                                 let tag = process.tag;
                                 process.kill();
                                 processes[handle.0] = None;
-                                event_sender.send(ApplicationEvent::ProcessExit {
-                                    tag,
-                                    success: false,
-                                })?;
+                                event_sender.send(ApplicationEvent::ProcessExit { tag })?;
                             }
                         }
                     }
@@ -1034,10 +1027,7 @@ fn run_server(args: Args, pipe_path: &[u16]) -> Result<(), AnyError> {
                             Ok(None) => (),
                             Ok(Some(buf)) => {
                                 if buf.as_bytes().is_empty() {
-                                    event_sender.send(ApplicationEvent::ProcessExit {
-                                        tag,
-                                        success: true,
-                                    })?;
+                                    event_sender.send(ApplicationEvent::ProcessExit { tag })?;
                                 } else {
                                     event_sender
                                         .send(ApplicationEvent::ProcessOutput { tag, buf })?;
@@ -1047,10 +1037,7 @@ fn run_server(args: Args, pipe_path: &[u16]) -> Result<(), AnyError> {
                                 process.stdout = None;
                                 process.kill();
                                 processes[i] = None;
-                                event_sender.send(ApplicationEvent::ProcessExit {
-                                    tag,
-                                    success: false,
-                                })?;
+                                event_sender.send(ApplicationEvent::ProcessExit { tag })?;
                             }
                         }
                     }
@@ -1342,4 +1329,3 @@ fn parse_console_events(
         }
     }
 }
-
