@@ -1,14 +1,10 @@
-use std::{cmp::Ordering, ops::Range};
-
 use crate::{
     buffer::BufferContent,
-    buffer_position::BufferRange,
+    buffer_position::{BufferPositionIndex, BufferRange},
     glob::{Glob, InvalidGlobError},
     pattern::{MatchResult, Pattern, PatternError, PatternState},
 };
 
-//mod builtin;
-
 const MAX_HIGHLIGHT_COUNT: usize = 2048;
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -24,9 +20,25 @@ pub enum TokenKind {
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
-struct Token {
+pub struct Token {
     pub kind: TokenKind,
-    pub range: Range<usize>,
+    pub from: BufferPositionIndex,
+    pub to: BufferPositionIndex,
+}
+impl Token {
+    #[inline]
+    pub fn contains(&self, column_byte_index: BufferPositionIndex) -> bool {
+        self.from <= column_byte_index && column_byte_index < self.to
+    }
+}
+impl Default for Token {
+    fn default() -> Self {
+        Self {
+            kind: TokenKind::Text,
+            from: 0,
+            to: 0,
+        }
+    }
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -43,7 +55,7 @@ impl Default for LineParseState {
 }
 
 pub struct Syntax {
-    raw_glob: Vec<u8>,
+    raw_glob: String,
     glob: Glob,
     rules: [Pattern; 7],
 }
@@ -53,7 +65,7 @@ impl Syntax {
         let mut text_pattern = Pattern::new();
         let _ = text_pattern.compile("%a{%w_}|_{%w_}");
         Self {
-            raw_glob: Vec::new(),
+            raw_glob: String::new(),
             glob: Glob::default(),
             rules: [
                 Pattern::new(),
@@ -67,9 +79,9 @@ impl Syntax {
         }
     }
 
-    pub fn set_glob(&mut self, pattern: &[u8]) -> Result<(), InvalidGlobError> {
+    pub fn set_glob(&mut self, pattern: &str) -> Result<(), InvalidGlobError> {
         self.raw_glob.clear();
-        self.raw_glob.extend_from_slice(pattern);
+        self.raw_glob.push_str(pattern);
         self.glob.compile(pattern)
     }
 
@@ -92,11 +104,12 @@ impl Syntax {
             LineParseState::Dirty => unreachable!(),
             LineParseState::Finished => (),
             LineParseState::Unfinished(kind, state) => {
-                match self.rules[kind as usize].matches_with_state(line, &state) {
+                match self.rules[kind as usize].matches_with_state(line, state) {
                     MatchResult::Ok(len) => {
                         tokens.push(Token {
                             kind,
-                            range: 0..len,
+                            from: 0,
+                            to: len as _,
                         });
                         index += len;
                     }
@@ -104,7 +117,8 @@ impl Syntax {
                     MatchResult::Pending(state) => {
                         tokens.push(Token {
                             kind,
-                            range: 0..line_len,
+                            from: 0,
+                            to: line_len as _,
                         });
                         return LineParseState::Unfinished(kind, state);
                     }
@@ -146,7 +160,8 @@ impl Syntax {
                     MatchResult::Pending(state) => {
                         tokens.push(Token {
                             kind,
-                            range: index..line_len,
+                            from: index as _,
+                            to: line_len as _,
                         });
                         return LineParseState::Unfinished(kind, state);
                     }
@@ -175,7 +190,8 @@ impl Syntax {
 
             tokens.push(Token {
                 kind,
-                range: from..index,
+                from: from as _,
+                to: index as _,
             });
         }
 
@@ -197,7 +213,7 @@ impl SyntaxCollection {
         Self { syntaxes }
     }
 
-    pub fn find_handle_by_path(&self, path: &[u8]) -> Option<SyntaxHandle> {
+    pub fn find_handle_by_path(&self, path: &str) -> Option<SyntaxHandle> {
         let mut iter = self.syntaxes.iter().enumerate();
         iter.next();
         for (i, syntax) in iter {
@@ -238,7 +254,7 @@ pub enum HighlightResult {
 pub struct HighlightedBuffer {
     highlighted_len: usize,
     lines: Vec<HighlightedLine>,
-    dirty_line_indexes: Vec<usize>,
+    dirty_line_indexes: Vec<BufferPositionIndex>,
 }
 
 impl HighlightedBuffer {
@@ -268,7 +284,7 @@ impl HighlightedBuffer {
         let insert_line_count = range.to.line_index - range.from.line_index;
         if insert_line_count > 0 {
             let previous_highlighted_len = self.highlighted_len;
-            self.highlighted_len += insert_line_count;
+            self.highlighted_len += insert_line_count as usize;
             if self.highlighted_len > self.lines.len() {
                 for line in &mut self.lines[previous_highlighted_len..] {
                     line.parse_state = LineParseState::Dirty;
@@ -282,7 +298,8 @@ impl HighlightedBuffer {
             }
 
             let insert_index = range.from.line_index + 1;
-            self.lines[insert_index..self.highlighted_len].rotate_right(insert_line_count);
+            self.lines[insert_index as usize..self.highlighted_len as usize]
+                .rotate_right(insert_line_count as _);
 
             for index in &mut self.dirty_line_indexes {
                 if insert_index <= *index {
@@ -291,18 +308,18 @@ impl HighlightedBuffer {
             }
         }
 
-        self.lines[range.from.line_index].parse_state = LineParseState::Dirty;
+        self.lines[range.from.line_index as usize].parse_state = LineParseState::Dirty;
         self.dirty_line_indexes.push(range.from.line_index);
     }
 
     pub fn on_delete(&mut self, range: BufferRange) {
-        self.lines[range.from.line_index].parse_state = LineParseState::Dirty;
+        self.lines[range.from.line_index as usize].parse_state = LineParseState::Dirty;
 
         let delete_line_count = range.to.line_index - range.from.line_index;
         if delete_line_count > 0 {
-            self.highlighted_len -= delete_line_count;
+            self.highlighted_len -= delete_line_count as usize;
             let delete_index = range.from.line_index + 1;
-            self.lines[delete_index..].rotate_left(delete_line_count);
+            self.lines[delete_index as usize..].rotate_left(delete_line_count as _);
 
             for index in &mut self.dirty_line_indexes {
                 if range.to.line_index <= *index {
@@ -330,7 +347,7 @@ impl HighlightedBuffer {
         let mut last_dirty_index = usize::MAX;
 
         let mut previous_parse_state = match index.checked_sub(1) {
-            Some(i) => self.lines[i].parse_state,
+            Some(i) => self.lines[i as usize].parse_state,
             None => LineParseState::Finished,
         };
 
@@ -340,16 +357,16 @@ impl HighlightedBuffer {
             let dirty_index = self.dirty_line_indexes[i];
             i += 1;
 
-            if dirty_index < index || dirty_index == last_dirty_index {
+            if dirty_index < index || dirty_index == last_dirty_index as _ {
                 continue;
             }
 
             index = dirty_index;
-            last_dirty_index = dirty_index;
+            last_dirty_index = dirty_index as _;
 
-            while index < self.highlighted_len {
-                let bline = buffer.line_at(index).as_str();
-                let hline = &mut self.lines[index];
+            while index < self.highlighted_len as _ {
+                let bline = buffer.line_at(index as _).as_str();
+                let hline = &mut self.lines[index as usize];
 
                 let previous_state = hline.parse_state;
                 previous_parse_state =
@@ -378,23 +395,11 @@ impl HighlightedBuffer {
         HighlightResult::Complete
     }
 
-    pub fn find_token_kind_at(&self, line_index: usize, char_index: usize) -> TokenKind {
-        if line_index >= self.highlighted_len {
-            return TokenKind::Text;
-        }
-
-        let tokens = &self.lines[line_index].tokens;
-        match tokens.binary_search_by(|t| {
-            if char_index < t.range.start {
-                Ordering::Greater
-            } else if char_index >= t.range.end {
-                Ordering::Less
-            } else {
-                Ordering::Equal
-            }
-        }) {
-            Ok(index) => tokens[index].kind,
-            Err(_) => TokenKind::Text,
+    pub fn line_tokens(&self, line_index: usize) -> &[Token] {
+        if line_index < self.highlighted_len {
+            &self.lines[line_index].tokens
+        } else {
+            &[]
         }
     }
 }
@@ -403,13 +408,20 @@ impl HighlightedBuffer {
 mod tests {
     use super::*;
 
+    use std::ops::Range;
+
     use crate::buffer_position::BufferPosition;
 
     fn assert_next_token<'a, I>(iter: &mut I, kind: TokenKind, range: Range<usize>)
     where
         I: Iterator<Item = &'a Token>,
     {
-        assert_eq!(Some(Token { kind: kind, range }), iter.next().cloned(),);
+        let expect = Some(Token {
+            kind,
+            from: range.start as _,
+            to: range.end as _,
+        });
+        assert_eq!(expect, iter.next().cloned());
     }
 
     fn highlighted_tokens<'a>(
@@ -423,7 +435,7 @@ mod tests {
 
     fn assert_token(slice: &str, kind: TokenKind, line: &str, token: &Token) {
         assert_eq!(kind, token.kind);
-        assert_eq!(slice, &line[token.range.clone()]);
+        assert_eq!(slice, &line[token.from as usize..token.to as usize]);
     }
 
     #[test]
