@@ -8,20 +8,20 @@ use crate::{
     editor::Editor,
     editor_utils::MessageKind,
     mode::ModeKind,
-    syntax::{HighlightedBuffer, TokenKind},
+    syntax::{HighlightedBuffer, Token, TokenKind},
     theme::Color,
 };
 
-pub const ENTER_ALTERNATE_BUFFER_CODE: &[u8] = b"\x1b[?1049h";
-pub const EXIT_ALTERNATE_BUFFER_CODE: &[u8] = b"\x1b[?1049l";
-pub const HIDE_CURSOR_CODE: &[u8] = b"\x1b[?25l";
-pub const SHOW_CURSOR_CODE: &[u8] = b"\x1b[?25h";
-pub const RESET_STYLE_CODE: &[u8] = b"\x1b[0;49m";
-pub const MODE_256_COLORS_CODE: &[u8] = b"\x1b[=19h";
-pub const BEGIN_TITLE_CODE: &[u8] = b"\x1b]0;";
-pub const END_TITLE_CODE: &[u8] = b"\x07";
+pub static ENTER_ALTERNATE_BUFFER_CODE: &[u8] = b"\x1b[?1049h";
+pub static EXIT_ALTERNATE_BUFFER_CODE: &[u8] = b"\x1b[?1049l";
+pub static HIDE_CURSOR_CODE: &[u8] = b"\x1b[?25l";
+pub static SHOW_CURSOR_CODE: &[u8] = b"\x1b[?25h";
+pub static RESET_STYLE_CODE: &[u8] = b"\x1b[0;49m";
+pub static MODE_256_COLORS_CODE: &[u8] = b"\x1b[=19h";
+pub static BEGIN_TITLE_CODE: &[u8] = b"\x1b]0;";
+pub static END_TITLE_CODE: &[u8] = b"\x07";
 
-const TOO_LONG_PREFIX: &[u8] = b"...";
+static TOO_LONG_PREFIX: &[u8] = b"...";
 
 #[inline]
 pub fn clear_line(buf: &mut Vec<u8>) {
@@ -133,14 +133,6 @@ impl<'a> View<'a> {
 }
 
 fn draw_buffer(buf: &mut Vec<u8>, editor: &Editor, view: &View, has_focus: bool) {
-    #[derive(Clone, Copy, PartialEq, Eq)]
-    enum DrawState {
-        Token(TokenKind),
-        Selection(TokenKind),
-        Highlight,
-        Cursor,
-    }
-
     let mut char_buf = [0; std::mem::size_of::<char>()];
 
     let cursor_color = if has_focus {
@@ -155,11 +147,8 @@ fn draw_buffer(buf: &mut Vec<u8>, editor: &Editor, view: &View, has_focus: bool)
         editor.theme.inactive_cursor
     };
 
-    let mut text_color = editor.theme.token_text;
-
     move_cursor_to(buf, 0, 0);
     set_background_color(buf, editor.theme.background);
-    set_foreground_color(buf, text_color);
     set_not_underlined(buf);
 
     let cursors = &view.cursors[..];
@@ -239,30 +228,48 @@ fn draw_buffer(buf: &mut Vec<u8>, editor: &Editor, view: &View, has_focus: bool)
         .skip(view.scroll.1 as _)
         .take(view.size.1 as _)
     {
+        #[derive(Clone, Copy, PartialEq, Eq)]
+        enum DrawState {
+            Token(TokenKind),
+            Selection(TokenKind),
+            Highlight,
+            Cursor,
+        }
+
         lines_drawn_count += 1;
 
+        let line = line.as_str();
         let mut draw_state = DrawState::Token(TokenKind::Text);
         let mut was_inside_diagnostic_range = false;
         let mut x = 0;
+        let mut last_line_token = Token::default();
+        let mut line_tokens = highlighted_buffer.line_tokens(line_index).iter();
 
         set_foreground_color(buf, editor.theme.token_text);
 
-        let line = line.as_str();
         for (char_index, c) in line.char_indices().chain(iter::once((line.len(), '\n'))) {
             if char_index < view.scroll.0 as _ {
                 continue;
             }
 
             let buf_len = buf.len();
-            let char_position = BufferPosition::line_col(line_index, char_index);
+            let char_position = BufferPosition::line_col(line_index as _, char_index as _);
 
             let token_kind = if c.is_ascii_whitespace() {
                 TokenKind::Whitespace
             } else {
-                highlighted_buffer.find_token_kind_at(line_index, char_index)
+                if !last_line_token.contains(char_index as _) {
+                    while let Some(token) = line_tokens.next() {
+                        if token.contains(char_index as _) {
+                            last_line_token = token.clone();
+                            break;
+                        }
+                    }
+                }
+                last_line_token.kind
             };
 
-            text_color = match token_kind {
+            let text_color = match token_kind {
                 TokenKind::Keyword => editor.theme.token_keyword,
                 TokenKind::Type => editor.theme.token_type,
                 TokenKind::Symbol => editor.theme.token_symbol,
@@ -346,12 +353,11 @@ fn draw_buffer(buf: &mut Vec<u8>, editor: &Editor, view: &View, has_focus: bool)
                     buf.push(editor.config.visual_space);
                 }
                 '\t' => {
-                    buf.push(editor.config.visual_tab_first);
                     let tab_size = editor.config.tab_size.get() as usize;
-                    let next_tab_stop = (tab_size - 1) - x % tab_size;
-                    x += next_tab_stop + 1;
+                    x += tab_size;
 
-                    for _ in 0..next_tab_stop {
+                    buf.push(editor.config.visual_tab_first);
+                    for _ in 0..tab_size - 1 {
                         buf.push(editor.config.visual_tab_repeat);
                     }
                 }
@@ -368,8 +374,9 @@ fn draw_buffer(buf: &mut Vec<u8>, editor: &Editor, view: &View, has_focus: bool)
             }
         }
 
+        set_background_color(buf, editor.theme.background);
+
         if x < view.size.0 as _ {
-            set_background_color(buf, editor.theme.background);
             clear_until_new_line(buf);
         }
 
@@ -583,7 +590,7 @@ fn draw_statusbar(buf: &mut Vec<u8>, editor: &Editor, view: &View, has_focus: bo
     match view.buffer {
         Some(buffer) => {
             buffer_needs_save = buffer.needs_save();
-            buffer_path = buffer.path().to_str().unwrap_or("<no path>");
+            buffer_path = buffer.path.to_str().unwrap_or("<no path>");
         }
         None => {
             buffer_needs_save = false;
